"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class ControlValve1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:ControlValve:1' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'ControlValve1'

    SERVICE_DEFAULT_VARIABLES = {
        "ControlMode": { "data_type": "string", "default": "CLOSED", "allowed_list": "['OPEN', 'CLOSED', 'AUTO']"},
        "MaxPosition": { "data_type": "ui1", "default": "100", "allowed_list": None},
        "MinPosition": { "data_type": "ui1", "default": "0", "allowed_list": None},
        "PositionStatus": { "data_type": "ui1", "default": "0", "allowed_list": None},
        "PositionTarget": { "data_type": "ui1", "default": "0", "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {}

    def action_GetMinMax(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetMinMax action.

            :returns: "CurrentMinPosition", "CurrentMaxPosition"
        """
        arguments = { }

        out_params = self.call_action("GetMinMax", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentMinPosition", "CurrentMaxPosition",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetMode(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetMode action.

            :returns: "CurrentControlMode"
        """
        arguments = { }

        out_params = self.call_action("GetMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentControlMode",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetPosition(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetPosition action.

            :returns: "CurrentPositionStatus"
        """
        arguments = { }

        out_params = self.call_action("GetPosition", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentPositionStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetPositionTarget(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetPositionTarget action.

            :returns: "CurrentPositionTarget"
        """
        arguments = { }

        out_params = self.call_action("GetPositionTarget", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentPositionTarget",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SetMinMax(self, NewMinPosition, NewMaxPosition, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetMinMax action.
        """
        arguments = {
            "NewMinPosition": NewMinPosition,
            "NewMaxPosition": NewMaxPosition,
        }

        self.call_action("SetMinMax", arguments=arguments, aspects=aspects)

        return

    def action_SetMode(self, NewControlMode, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetMode action.
        """
        arguments = {
            "NewControlMode": NewControlMode,
        }

        self.call_action("SetMode", arguments=arguments, aspects=aspects)

        return

    def action_SetPosition(self, NewPositionTarget, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetPosition action.
        """
        arguments = {
            "NewPositionTarget": NewPositionTarget,
        }

        self.call_action("SetPosition", arguments=arguments, aspects=aspects)

        return



"""
.. module:: dnsconst
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Contains the constants that are used in association with multicast DNS.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

import enum
import re

from akit.xenum import DocIntEnum

DNS_SIZE_SIGNED_CHAR = 1
DNS_SIZE_SIGNED_SHORT = 2
DNS_SIZE_SIGNED_INTEGER = 4

DNS_SIZE_UNSIGNED_CHAR = 1
DNS_SIZE_UNSIGNED_SHORT = 2
DNS_SIZE_UNSIGNED_INTEGER = 4

DNS_PACK_FORMAT_SHORT = "!H"
DNS_PACK_FORMAT_INTEGER = "!I"

@enum.unique
class DnsLiftimePercent(enum.IntEnum):
    """
        Enumeration for the DNS lifetime percentage states.
    """
    Expired = 100
    Stale = 50
    Refresh = 75

@enum.unique
class DnsQuestionType(enum.IntEnum):
    """
        The MDNS question type.

        "QU" - questions requesting unicast responses.
        "QM" - questions requesting multicast responses.
    """
    QU=1
    QM=2

@enum.unique
class DnsRecordClass(enum.IntEnum):
    """
        Enumeration for the DNS record class.
    """
    IN = 1 # The internet
    CS = 2 # The CSNET class
    CH = 3 # The CHAOS class
    HS = 4 # Hesiod

    NONE = 254
    ANY = 255
    MASK = 0x7FFF
    UNIQUE = 0x8000

@enum.unique
class DnsRecordType(enum.IntEnum):
    """
        Enumeration for the DNS record types.
    """
    A = 1 #,              "RFC 1035 - Address Record"
    NS = 2 #,             "RFC 1035 - Name server record"
    MD = 3 #,             "RFC 833 - Mail Destination"
    MF = 4 #,             "RFC 833 - Mail Forwarder"
    CNAME = 5 #,          "RFC 1035 - Conanical record name"
    SOA = 6 #,            "RFC 1035,2308 - Start of [a zone of] authority record"
    MB = 7 #,             "RFC 833 - Mailbox Binding"
    MG = 8 #,             "RFC 833 - Mail Group"
    MR = 9 #,             "RFC 833 - Mail Rename"
    NULL = 10 #,          "RFC 1035 - Null record or placeholder"
    WKS = 11 #,           "RFC 1035 - Well Known Service"
    PTR = 12 #,           "RFC 1035 - PTR Resource Record"
    HINFO = 13 #,         "RFC 8482 - Host Information"
    MINFO = 14 #,         "RFC 1035 - Mail Information Record"
    MX = 15 #,            "RFC 1035,7505 - Mail exchange record"
    TXT = 16 #,           "RFC 1035 - Text record"
    RP = 17 #,            "RFC 1183 - Responsible Person"
    AFSDB = 18 #,         "RFC 1183 - AFS database record"
    SIG = 24 #,           "RFC 2535 - Signature"
    KEY = 25 #,           "RFC 2535,2930 - Key record"
    AAAA = 28 #,          "RFC 3596 - IPv6 Address Record"
    LOC = 29 #,           "RFC 1876 - Location record"
    SRV = 33 #,           "RFC 2782 - Service locator"
    NAPTR = 35 #,         "RFC 3403 - Naming Authority Pointer"
    KX = 36 #,            "RFC 2230 - Key Exchanger record"
    CERT = 37 #,          "RFC 4398 - Certificate record"
    DNAME = 39 #,         "RFC 6672 - Delegation name record"
    APL = 42 #,           "RFC 3123 - Address Prefix List"
    DS = 43 #,            "RFC 4034 - Delegation signer"
    SSHFP = 44 #,         "RFC 4255 - SSH Public Key Fingerprint"
    IPSECKEY = 45 #,      "RFC 4025 - IPsec Key"
    RRSIG = 46 #,         "RFC 4034 - DNSSEC signature"
    NSEC = 47 #,          "RFC 4034 - Next Secure record"
    DNSKEY = 48 #,        "RFC 4034 - DNS Key record"
    DHCID = 49 #,         "RFC 4701 - DHCP identifier"
    NSEC3 = 50 #,         "RFC 5155 - Next Secure record version 3"
    NSEC3PARAM = 51 #,    "RFC 5155 - NSEC3 parameters"
    TLSA = 52 #,          "RFC 6698 - TLSA certificate association"
    SMIMEA = 53 #,        "RFC 8162 - S/MIME cert association"
    HIP = 55 #,           "RFC 8005 - Host Identity Protocol"
    CDS = 59 #,           "RFC 7344 - Child copy of DS record, for transfer to parent"
    CDNSKEY = 60 #,       "RFC 7344 - Child copy of DNSKEY record, for transfer to parent"
    OPENPGPKEY = 61 #,    "RFC 7929 - OpenPGP public key record"
    CSYNC = 62 #,         "RFC 7477 - Child-to-Parent Synchronization"
    ZONEMD = 63 # 
    SVCB = 64 #,          "RFC ???? - Service Binding"
    HTTPS = 65 #,         "RFC ???? - HTTPS Binding"
    EUI48 = 108 #,        "RFC 7043 - MAC address (EUI-48)"
    EUI64 = 109 #,        "RFC 7043 - MAC address (EUI-64)"
    TKEY = 249 #,         "RFC 2930 - Transaction Key record"
    TSIG = 250 #,         "RFC 2845 - Transaction Signature"
    AXFR = 252 #,         "RFC 1035 - Zone transfer request"
    MAILB = 253 #,        "RFC 1035 - Request for mailbox related methods"
    MAILA = 254 #,        "RFC 1035 - Request for mail agent related records"
    ANY = 255 #,          "RFC 1035 - Any class query class"
    URI = 256 #,          "RFC 7553 - Uniform Resource Identifier"
    CAA = 257 #,          "RFC 6844 - Certification Authority Authorization"
    TA = 32768 #,         "RFC ???? - DNSSEC Trust Authorities"
    DLV = 32769 #,        "RFC 4431 - DNSSEC Lookaside Validation record"

class DnsFlags(enum.IntEnum):
    """
        Enumeration for the DNS record flags.
    """
    FLAGS_AA = 0x0400  # Authoritative answer
    FLAGS_TC = 0x0200  # Truncated
    FLAGS_RD = 0x0100  # Recursion desired
    FLAGS_RA = 0x8000  # Recursion available

    FLAGS_Z =  0x0040   # Zero
    FLAGS_AD = 0x0020  # Authentic data
    FLAGS_CD = 0x0010  # Checking disabled

class DnsResponseFlags(enum.IntEnum):
    """
        Enumeration for the DNS response flags.
    """
    FLAGS_QR_MASK = 0x8000  # query response mask
    FLAGS_QR_QUERY = 0x0000  # query
    FLAGS_QR_RESPONSE = 0x8000  # response

EXPIRE_FULL_TIME_PERCENT = 100
EXPIRE_STALE_TIME_PERCENT = 50
EXPIRE_REFRESH_TIME_PERCENT = 75

HAS_A_TO_Z = re.compile(r'[A-Za-z]')
HAS_ONLY_A_TO_Z_NUM_HYPHEN = re.compile(r'^[A-Za-z0-9\-]+$')
HAS_ONLY_A_TO_Z_NUM_HYPHEN_UNDERSCORE = re.compile(r'^[A-Za-z0-9\-\_]+$')
HAS_ASCII_CONTROL_CHARS = re.compile(r'[\x00-\x1f\x7f]')

MAX_MSG_TYPICAL = 1460  # unused
MAX_MSG_ABSOLUTE = 8966

class DnsKnownServiceTypes:
    AIRTUNES = "AirTunes Remote Audio"
    AIRPLAY = "_airplay._tcp"
    MICROSOFT_WINDOWS_NETWORK = "Microsoft Windows Network"
    SFTP = "SFTP File Transfer"
    SONOS = "_sonos._tcp"
    SPOTIFY_CONNECT = "_spotify-connect._tcp"
    SSH = "SSH Remote Terminal"
    VNC = "VNC Remote Access"


"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class ContentDirectory4ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:ContentDirectory:4' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'ContentDirectory4'

    SERVICE_DEFAULT_VARIABLES = {
        "DeviceModeStatus": { "data_type": "string", "default": None, "allowed_list": None},
        "FeatureList": { "data_type": "string", "default": None, "allowed_list": None},
        "PermissionsInfo": { "data_type": "string", "default": None, "allowed_list": None},
        "SearchCapabilities": { "data_type": "string", "default": None, "allowed_list": None},
        "ServiceResetToken": { "data_type": "string", "default": None, "allowed_list": None},
        "SortCapabilities": { "data_type": "string", "default": None, "allowed_list": None},
        "SortExtensionCapabilities": { "data_type": "string", "default": None, "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {
        "ContainerUpdateIDs": { "data_type": "string", "default": None, "allowed_list": None},
        "DeviceMode": { "data_type": "string", "default": None, "allowed_list": None},
        "LastChange": { "data_type": "string", "default": None, "allowed_list": None},
        "SystemUpdateID": { "data_type": "ui4", "default": None, "allowed_list": None},
        "TransferIDs": { "data_type": "string", "default": None, "allowed_list": None},
        "TransformStatus": { "data_type": "string", "default": None, "allowed_list": None},
    }

    def action_Browse(self, ObjectID, BrowseFlag, Filter, StartingIndex, RequestedCount, SortCriteria, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the Browse action.

            :returns: "Result", "NumberReturned", "TotalMatches", "UpdateID"
        """
        arguments = {
            "ObjectID": ObjectID,
            "BrowseFlag": BrowseFlag,
            "Filter": Filter,
            "StartingIndex": StartingIndex,
            "RequestedCount": RequestedCount,
            "SortCriteria": SortCriteria,
        }

        out_params = self.call_action("Browse", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Result", "NumberReturned", "TotalMatches", "UpdateID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_CancelDeviceMode(self, DeviceModeID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the CancelDeviceMode action.
        """
        arguments = {
            "DeviceModeID": DeviceModeID,
        }

        self.call_action("CancelDeviceMode", arguments=arguments, aspects=aspects)

        return

    def action_CancelTransformTask(self, TransformTaskID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the CancelTransformTask action.
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
        }

        self.call_action("CancelTransformTask", arguments=arguments, aspects=aspects)

        return

    def action_CreateObject(self, ContainerID, Elements, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the CreateObject action.

            :returns: "ObjectID", "Result"
        """
        arguments = {
            "ContainerID": ContainerID,
            "Elements": Elements,
        }

        out_params = self.call_action("CreateObject", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("ObjectID", "Result",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_CreateReference(self, ContainerID, ObjectID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the CreateReference action.

            :returns: "NewID"
        """
        arguments = {
            "ContainerID": ContainerID,
            "ObjectID": ObjectID,
        }

        out_params = self.call_action("CreateReference", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("NewID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_DeleteResource(self, ResourceURI, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the DeleteResource action.
        """
        arguments = {
            "ResourceURI": ResourceURI,
        }

        self.call_action("DeleteResource", arguments=arguments, aspects=aspects)

        return

    def action_DestroyObject(self, ObjectID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the DestroyObject action.
        """
        arguments = {
            "ObjectID": ObjectID,
        }

        self.call_action("DestroyObject", arguments=arguments, aspects=aspects)

        return

    def action_EvaluateTransforms(self, TransformResourceDesc, TransformSettings, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the EvaluateTransforms action.

            :returns: "EvaluationResult"
        """
        arguments = {
            "TransformResourceDesc": TransformResourceDesc,
            "TransformSettings": TransformSettings,
        }

        out_params = self.call_action("EvaluateTransforms", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("EvaluationResult",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_ExportResource(self, SourceURI, DestinationURI, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ExportResource action.

            :returns: "TransferID"
        """
        arguments = {
            "SourceURI": SourceURI,
            "DestinationURI": DestinationURI,
        }

        out_params = self.call_action("ExportResource", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransferID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_ExtendDeviceMode(self, DeviceModeID, DeviceModeRequest, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ExtendDeviceMode action.

            :returns: "DeviceModeStatus"
        """
        arguments = {
            "DeviceModeID": DeviceModeID,
            "DeviceModeRequest": DeviceModeRequest,
        }

        out_params = self.call_action("ExtendDeviceMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DeviceModeStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_FreeFormQuery(self, ContainerID, CDSView, QueryRequest, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the FreeFormQuery action.

            :returns: "QueryResult", "UpdateID"
        """
        arguments = {
            "ContainerID": ContainerID,
            "CDSView": CDSView,
            "QueryRequest": QueryRequest,
        }

        out_params = self.call_action("FreeFormQuery", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("QueryResult", "UpdateID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAllAvailableTransforms(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAllAvailableTransforms action.

            :returns: "AllAvailableTransforms"
        """
        arguments = { }

        out_params = self.call_action("GetAllAvailableTransforms", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AllAvailableTransforms",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAllowedTransforms(self, TransformResourceObjectDesc, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAllowedTransforms action.

            :returns: "AllowedTransforms"
        """
        arguments = {
            "TransformResourceObjectDesc": TransformResourceObjectDesc,
        }

        out_params = self.call_action("GetAllowedTransforms", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AllowedTransforms",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetCurrentTransformStatusList(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetCurrentTransformStatusList action.

            :returns: "TransformStatus"
        """
        arguments = { }

        out_params = self.call_action("GetCurrentTransformStatusList", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransformStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDeviceMode(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDeviceMode action.

            :returns: "DeviceMode"
        """
        arguments = { }

        out_params = self.call_action("GetDeviceMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DeviceMode",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDeviceModeStatus(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDeviceModeStatus action.

            :returns: "DeviceModeStatus"
        """
        arguments = { }

        out_params = self.call_action("GetDeviceModeStatus", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DeviceModeStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFeatureList(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFeatureList action.

            :returns: "FeatureList"
        """
        arguments = { }

        out_params = self.call_action("GetFeatureList", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("FeatureList",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFreeFormQueryCapabilities(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFreeFormQueryCapabilities action.

            :returns: "FFQCapabilities"
        """
        arguments = { }

        out_params = self.call_action("GetFreeFormQueryCapabilities", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("FFQCapabilities",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetPermissionsInfo(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetPermissionsInfo action.

            :returns: "PermissionsInfo"
        """
        arguments = { }

        out_params = self.call_action("GetPermissionsInfo", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("PermissionsInfo",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSearchCapabilities(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSearchCapabilities action.

            :returns: "SearchCaps"
        """
        arguments = { }

        out_params = self.call_action("GetSearchCapabilities", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("SearchCaps",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetServiceResetToken(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetServiceResetToken action.

            :returns: "ResetToken"
        """
        arguments = { }

        out_params = self.call_action("GetServiceResetToken", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("ResetToken",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSortCapabilities(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSortCapabilities action.

            :returns: "SortCaps"
        """
        arguments = { }

        out_params = self.call_action("GetSortCapabilities", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("SortCaps",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSortExtensionCapabilities(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSortExtensionCapabilities action.

            :returns: "SortExtensionCaps"
        """
        arguments = { }

        out_params = self.call_action("GetSortExtensionCapabilities", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("SortExtensionCaps",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSystemUpdateID(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSystemUpdateID action.

            :returns: "Id"
        """
        arguments = { }

        out_params = self.call_action("GetSystemUpdateID", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Id",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTransferProgress(self, TransferID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTransferProgress action.

            :returns: "TransferStatus", "TransferLength", "TransferTotal"
        """
        arguments = {
            "TransferID": TransferID,
        }

        out_params = self.call_action("GetTransferProgress", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransferStatus", "TransferLength", "TransferTotal",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTransformTaskResult(self, TransformTaskID, TransformTaskResultFilter, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTransformTaskResult action.

            :returns: "TransformTaskResult"
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
            "TransformTaskResultFilter": TransformTaskResultFilter,
        }

        out_params = self.call_action("GetTransformTaskResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransformTaskResult",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTransforms(self, TransformTaskID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTransforms action.

            :returns: "CurrentTransformSettings"
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
        }

        out_params = self.call_action("GetTransforms", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentTransformSettings",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_ImportResource(self, SourceURI, DestinationURI, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ImportResource action.

            :returns: "TransferID"
        """
        arguments = {
            "SourceURI": SourceURI,
            "DestinationURI": DestinationURI,
        }

        out_params = self.call_action("ImportResource", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransferID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_MoveObject(self, ObjectID, NewParentID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the MoveObject action.

            :returns: "NewObjectID"
        """
        arguments = {
            "ObjectID": ObjectID,
            "NewParentID": NewParentID,
        }

        out_params = self.call_action("MoveObject", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("NewObjectID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_PauseTransformTask(self, TransformTaskID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the PauseTransformTask action.
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
        }

        self.call_action("PauseTransformTask", arguments=arguments, aspects=aspects)

        return

    def action_RequestDeviceMode(self, CPID, DeviceModeRequest, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the RequestDeviceMode action.

            :returns: "DeviceModeID", "DeviceModeStatus"
        """
        arguments = {
            "CPID": CPID,
            "DeviceModeRequest": DeviceModeRequest,
        }

        out_params = self.call_action("RequestDeviceMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DeviceModeID", "DeviceModeStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_ResumeTransformTask(self, TransformTaskID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ResumeTransformTask action.
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
        }

        self.call_action("ResumeTransformTask", arguments=arguments, aspects=aspects)

        return

    def action_RollbackTransformTask(self, TransformTaskID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the RollbackTransformTask action.
        """
        arguments = {
            "TransformTaskID": TransformTaskID,
        }

        self.call_action("RollbackTransformTask", arguments=arguments, aspects=aspects)

        return

    def action_Search(self, ContainerID, SearchCriteria, Filter, StartingIndex, RequestedCount, SortCriteria, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the Search action.

            :returns: "Result", "NumberReturned", "TotalMatches", "UpdateID"
        """
        arguments = {
            "ContainerID": ContainerID,
            "SearchCriteria": SearchCriteria,
            "Filter": Filter,
            "StartingIndex": StartingIndex,
            "RequestedCount": RequestedCount,
            "SortCriteria": SortCriteria,
        }

        out_params = self.call_action("Search", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Result", "NumberReturned", "TotalMatches", "UpdateID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_StartTransformTask(self, TransformResourceDesc, TransformSettings, TransformOverwrite, TransformRollback, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StartTransformTask action.

            :returns: "TransformTaskID"
        """
        arguments = {
            "TransformResourceDesc": TransformResourceDesc,
            "TransformSettings": TransformSettings,
            "TransformOverwrite": TransformOverwrite,
            "TransformRollback": TransformRollback,
        }

        out_params = self.call_action("StartTransformTask", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TransformTaskID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_StopTransferResource(self, TransferID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StopTransferResource action.
        """
        arguments = {
            "TransferID": TransferID,
        }

        self.call_action("StopTransferResource", arguments=arguments, aspects=aspects)

        return

    def action_UpdateObject(self, ObjectID, CurrentTagValue, NewTagValue, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the UpdateObject action.
        """
        arguments = {
            "ObjectID": ObjectID,
            "CurrentTagValue": CurrentTagValue,
            "NewTagValue": NewTagValue,
        }

        self.call_action("UpdateObject", arguments=arguments, aspects=aspects)

        return



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"


import os

from datetime import datetime

import click

HELP_CREDENTIALS = "The full path of the credentials file to use for the testrun."
HELP_LANDSCAPE = "The full path of the landscape file to verify."
HELP_RUNTIME = "The full path of the runtime file to use for the testrun."

@click.group("landscape", help="Contains commands that are used to perform operations on the test landscape.")
def group_akit_landscape():
    return

@click.command("verify")
@click.option("--credentials", "credentials_file", default=None, required=False, help=HELP_CREDENTIALS)
@click.option("--landscape", "landscape_file", default=None, required=False, help=HELP_LANDSCAPE)
@click.option("--runtime", "runtime_file", default=None, required=False, help=HELP_RUNTIME)
def command_akit_landscape_verify(credentials_file, landscape_file, runtime_file):
    
    # pylint: disable=unused-import,import-outside-toplevel

    # We do the imports of the automation framework code inside the action functions because
    # we don't want to startup loggin and the processing of inputs and environment variables
    # until we have entered an action method.  Thats way we know how to setup the environment.

    # IMPORTANT: We need to load the context first because it will trigger the loading
    # of the default user configuration
    from akit.environment.context import Context
    from akit.environment.contextpaths import ContextPaths

    from akit.environment.variables import JOB_TYPES, AKIT_VARIABLES
    
    from akit.environment.optionoverrides import (
        override_config_credentials,
        override_config_landscape,
        override_config_runtime
    )

    ctx = Context()
    env = ctx.lookup("/environment")

    # We need to set the job type before we trigger activation.
    env["jobtype"] = JOB_TYPES.COMMAND

    # Activate the AutomationKit environment with characteristics of a console application
    import akit.activation.console

    from akit.xlogging.foundations import logging_initialize, getAutomatonKitLogger

    # Initialize logging
    logging_initialize()
    logger = getAutomatonKitLogger()

    if credentials_file is not None:
        override_config_credentials(credentials_file)

    if landscape_file is not None:
        override_config_landscape(landscape_file)

    if runtime_file is not None:
        override_config_runtime(runtime_file)

    from akit.interop.landscaping.landscape import Landscape, startup_landscape
    from akit.interop.landscaping.landscapedevice import LandscapeDevice

    lscape: Landscape = startup_landscape(include_ssh=True, include_upnp=True)

    upnp_device_configs = lscape.get_upnp_device_configs()
    if len(upnp_device_configs):
        print("======================= UPNP DEVICES =======================")

        for dev in upnp_device_configs:
            skip_dev = True if "skip" in dev and dev["skip"] else False

            upnp_info = dev["upnp"]
            usn = upnp_info["USN"]
            modelName = upnp_info["modelName"]
            modelNumber = upnp_info["modelNumber"]

            status = "Down"
            lscape_dev: LandscapeDevice = lscape.lookup_device_by_identity(usn)
            if lscape_dev is not None:
                status = lscape_dev.verify_status()

            dev_info_lines = [
                "    Model: {} - {}".format(modelName, modelNumber),
                "      USN: {}".format(usn),
                "     Skip: {}".format(skip_dev),
                "   Status: {}".format(status)
            ]
            dev_info = os.linesep.join(dev_info_lines)

            print(dev_info)
            print("")

    ssh_devices_configs = lscape.get_ssh_device_configs(exclude_upnp=True)
    if len(ssh_devices_configs):
        print("======================= SSH DEVICES =======================")

        for dev in ssh_devices_configs:
            skip_dev = True if "skip" in dev and dev["skip"] else False

            host = dev["host"]

            status = "Down"
            lscape_dev: LandscapeDevice = lscape.lookup_device_by_identity(host)
            if lscape_dev is not None:
                status = lscape_dev.verify_status()

            dev_info_lines = [
                "    HOST: {}".format(host),
                "   Status: {}".format(status)
            ]
            dev_info = os.linesep.join(dev_info_lines)

            print(dev_info)
            print("")

    return

group_akit_landscape.add_command(command_akit_landscape_verify)



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from typing import List, Optional

import os

from akit.exceptions import AKitCommandError, AKitHTTPRequestError, TracebackFormatPolicy
from akit.xformatting import format_command_result
import requests

__traceback_format_policy__ = TracebackFormatPolicy.Hide

def raise_for_command_status(status: int, stdout: str, stderr: str, context:str, exp_status: Optional[List[int]]=None):
    """
        Raises an :class:`AKitCommandError` if the status code is not what was expected.
    """

    if exp_status is None:
        exp_status = [0]

    if status not in exp_status:
        errmsg = format_command_result(context, status, stdout, stderr, exp_status=exp_status)
        raise AKitCommandError(errmsg, status, stdout, stderr)

    return

def raise_for_http_status(context: str, response: requests.Response, details: Optional[dict]=None, allow_redirects: bool=False):
    """
        Raises an :class:`AKitHTTPRequestError` if an HTTP response error occured.
    """

    status_code = response.status_code
    method = response.request.method
    req_url = response.url

    if status_code >= 400 or (not allow_redirects and status_code >= 300):
        err_msg_lines = [
            context
        ]

        reason = response.reason

        # If we have `bytes` then we need to decode it
        if isinstance(reason, bytes):
            try:
                reason = reason.decode('utf-8')
            except UnicodeDecodeError:
                reason = reason.decode('iso-8859-1')

        if status_code < 400:
            # Client Error
            err_msg_lines.append("{} Redirect Error: {} for url: {} method: {}".format(
                status_code, reason, response.url, method))
        elif status_code < 500:
            # Client Error
            err_msg_lines.append("{} Client Error: {} for url: {} method: {}".format(
                status_code, reason, response.url, method))
        elif status_code >= 500 and status_code < 600:
            # Server Error
            err_msg_lines.append("{} Server Error: {} for url: {} method: {}".format(
                status_code, reason, response.url, method))
        else:
            err_msg_lines.append("{} UnExpected Error: {} for url: {} method: {}".format(
                status_code, reason, response.url, method))

        if details is not None:
            for dkey, dval in details.items():
                err_msg_lines.append("    {}: {}".format(dkey, dval))

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitHTTPRequestError(errmsg, req_url, status_code, reason)

    return




import os
import socket

import zeroconf

class MdnsServiceInfo:
    def __init__(self, serviceinfo: zeroconf.ServiceInfo):
        self._serviceinfo = serviceinfo

    @property
    def addresses(self):
        return self._serviceinfo.addresses

    @property
    def first_ipv4_address(self):
        fipv4_addr = None

        for addr in self._serviceinfo.addresses:
            if len(addr) == 4: 
                fipv4_addr = socket.inet_ntoa(addr)
                break

        return fipv4_addr

    @property
    def host_ttl(self):
        return self._serviceinfo.host_ttl

    @property
    def interface_index(self):
        return self._serviceinfo.interface_index

    @property
    def key(self):
        return self._serviceinfo.key
    
    @property
    def other_ttl(self):
        return self._serviceinfo.other_ttl
    
    @property
    def port(self):
        return self._serviceinfo.port

    @property
    def priority(self):
        return self._serviceinfo.priority
    
    @property
    def properties(self):
        return self._serviceinfo.properties
    
    @property
    def server(self):
        return self._serviceinfo.server
    
    @property
    def server_key(self):
        return self._serviceinfo.server_key

    @property
    def svc_name(self):
        return self._serviceinfo.name
    
    @property
    def svc_type(self):
        return self._serviceinfo.type

    @property
    def text(self):
        return self._serviceinfo.text
    
    @property
    def weight(self):
        return self._serviceinfo.weight

    @property
    def wrapped_serviceinfo(self):
        return self._serviceinfo

    def detail_lines(self):

        str_lines = [
            "svc_name: {}".format(self.svc_name),
            "svc_type: {}".format(self.svc_type),
            "ipv4_addr: {}".format(self.first_ipv4_address),
            "svc_type: {}".format(self.svc_type),
            "properties"
        ]

        svc_props = self.properties

        svc_prop_keys = [k for k in svc_props]
        svc_prop_keys.sort()

        for key in svc_prop_keys:
            val = svc_props[key]
            if isinstance(val, bytes):
                str_lines.append("    {}: {}".format(key, repr(val.decode())))
            else:
                str_lines.append("    {}: {}".format(key, repr(val)))

        return str_lines

    def __str__(self):
        
        str_lines = self.detail_lines()
        strval = os.linesep.join(str_lines)

        return strval



"""
.. module:: upnpfactory
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Module containing the :class:`UpnpFactory` class and associated diagnostic.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>

"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from typing import Union

from akit.exceptions import AKitSemanticError
from akit.extensible import generate_extension_key
from akit.compat import import_by_name

from akit.environment.variables import AKIT_VARIABLES

from akit.interop.upnp.devices.upnpembeddeddevice import UpnpEmbeddedDevice
from akit.interop.upnp.devices.upnprootdevice import UpnpRootDevice
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

from akit.interop.upnp.extensions import standard as standard_extensions

from akit.extensible import collect_extensions_under_code_container, collect_extensions_under_folder

class UpnpFactory:
    """
        The :class:`UpnpFactory` object is a singleton object that manages the cataloging and instantiation
        of code generated UPnP device and service extensions.  We utilize code generated device classes and
        service proxy classes because that makes each class of device and each method on those classes
        individually isolatable and debugable.
    """

    _instance = None
    _initialized = False

    def __new__(cls):
        """
            Constructs new instances of the UpnpFactory object from the :class:`UpnpDeviceFactory`.
            This is a singlton object that is used to register and instantiate UPNP devices based on
            their devicetype id.
        """
        if cls._instance is None:
            cls._instance = super(UpnpFactory, cls).__new__(cls)
        return cls._instance

    def __init__(self):
        """
            Initializes the Singleton initializer class
        """
        thisType = type(self)
        if not thisType._initialized:
            thisType._initialized = True

            self._dyn_embedded_device_registry = {}
            self._dyn_root_device_registry = {}
            self._dyn_service_registry = {}
            self._std_embedded_device_registry = {}
            self._std_root_device_registry = {}
            self._std_service_registry = {}

            dyn_ext_module = None
            dyn_ext_module_name = AKIT_VARIABLES.AKIT_UPNP_DYN_EXTENSIONS_MODULE
            if dyn_ext_module_name is not None:
                dyn_ext_module = import_by_name(dyn_ext_module_name)

            dyn_ext_folder = AKIT_VARIABLES.AKIT_UPNP_EXTENSIONS_INTEGRATION_BASE

            if dyn_ext_module is not None:
                self._scan_for_device_extensions_under_code_container(dyn_ext_module, self._dyn_root_device_registry)
                self._scan_for_service_extensions_under_code_container(dyn_ext_module, self._dyn_service_registry)
            elif dyn_ext_folder is not None:
                self._scan_for_device_extensions_under_folder(dyn_ext_folder, self._dyn_root_device_registry)
                self._scan_for_service_extensions_under_folder(dyn_ext_folder, self._dyn_service_registry)
            
            self._scan_for_device_extensions_under_code_container(standard_extensions, self._std_root_device_registry)
            self._scan_for_service_extensions_under_code_container(standard_extensions, self._std_service_registry)
        return

    def create_embedded_device_instance(self, manufacturer:str, modelNumber: str, modelDescription: str) -> UpnpEmbeddedDevice:
        """
            Method that is called to create an instance of an extension for a specific UPnP embedded device. If an
            embedded device extension is not found matching the specified parameters then a generic UpnpEmbeddedDevice
            instance is created and returned.

            :param manufacturer: The manufacturer associated with the embedded device.
            :param modelNumber: The model number associated with the embedded device.
            :param modelDescription: The model description associated with the embedded device.

            :returns: An instance of an embedded device extension of a specific manaufacturer or a default generic
                      UpnpEmbeddedDevice.
        """
        deviceClass = UpnpEmbeddedDevice
        extkey = generate_extension_key(manufacturer, modelNumber, modelDescription)
        if extkey in self._dyn_embedded_device_registry:
            deviceClass = self._dyn_embedded_device_registry[extkey]
        elif extkey in self._std_embedded_device_registry:
            deviceClass = self._std_embedded_device_registry[extkey]
        return deviceClass()

    def create_root_device_instance(self, manufacturer:str, modelNumber: str, modelDescription: str) -> UpnpRootDevice:
        """
            Method that is called to create an instance of an extension for a specific UPnP root device.  If a device
            extension is not found matching the specified parameters then a generic UpnpRootDevice instance is
            created and returned.

            :param manufacturer: The manufacturer associated with the root device.
            :param modelNumber: The model number associated with the root device.
            :param modelDescription: The model description associated with the root device.

            :returns: An instance of an root device extension of a specific manaufacturer or a default generic
                      UpnpRootDevice.
        """
        deviceClass = UpnpRootDevice

        if manufacturer is not None and modelNumber is not None and modelDescription is not None:
            extkey = generate_extension_key(manufacturer, modelNumber, modelDescription)
            if extkey in self._dyn_root_device_registry:
                deviceClass = self._dyn_root_device_registry[extkey]
            elif extkey in self._std_root_device_registry:
                deviceClass = self._std_root_device_registry[extkey]

        dev_inst = deviceClass(manufacturer, modelNumber, modelDescription)
        return dev_inst

    def create_service_instance(self, serviceManufacturer, serviceType) -> Union[UpnpServiceProxy, None]:
        """
            Method that is called to create an instance of an extension for a specific UPnP service proxy.  If a
            service proxy extension is not found matching the specified parameters then None is returned.

            :param manufacturer: The manufacturer associated with the root device.
            :param modelNumber: The model number associated with the root device.
            :param modelDescription: The model description associated with the root device.

            :returns: An instance of an root device extension of a specific manaufacturer or a default generic
                      UpnpRootDevice.
        """
        serviceInst = None
        if serviceType is not None:
            extkey = generate_extension_key(serviceManufacturer, serviceType)
            if extkey in self._dyn_service_registry:
                serviceClass = self._dyn_service_registry[extkey]
                serviceInst = serviceClass()
            elif extkey in self._std_service_registry:
                serviceClass = self._std_service_registry[extkey]
                serviceInst = serviceClass()
        return serviceInst

    def _register_root_device(self, extkey, extcls, device_table):
        """
            Method that registers the extension class of a specific type of root device.
        """
        if extkey not in device_table:
            device_table[extkey] = extcls
        else:
            raise AKitSemanticError("A root device extension with the key=%r was already registered. (%s)" % (extkey, extcls)) from None
        return

    def _register_service(self, extkey, extcls, service_table):
        """
            Method that registers the extension class of a specific type of service proxy.
        """
        if extkey not in service_table:
            service_table[extkey] = extcls
        else:
            service_table[extkey] = extcls
        return

    def _scan_for_device_extensions_under_code_container(self, container, device_table):
        """
            Method that scans a code container and its descendants for UpnpRootDevice objects.
        """
        extcoll = collect_extensions_under_code_container(container, UpnpRootDevice)
        for _, extcls in extcoll:
            if hasattr(extcls, "MANUFACTURER") and hasattr(extcls, "MODEL_NUMBER") and hasattr(extcls, "MODEL_DESCRIPTION"):
                extkey = generate_extension_key(getattr(extcls, "MANUFACTURER"),
                    getattr(extcls, "MODEL_NUMBER"), getattr(extcls, "MODEL_DESCRIPTION"))
                self._register_root_device(extkey, extcls, device_table)
        return

    def _scan_for_service_extensions_under_code_container(self, container, service_table):
        """
            Method that scans a code container and its descendants for UpnpServiceProxy objects.
        """
        extcoll = collect_extensions_under_code_container(container, UpnpServiceProxy)
        for _, extcls in extcoll:
            if (hasattr(extcls, "SERVICE_MANUFACTURER") and hasattr(extcls, "SERVICE_TYPE")):
                svc_manufacturer = getattr(extcls, "SERVICE_MANUFACTURER")
                svc_type = getattr(extcls, "SERVICE_TYPE")
                extkey = generate_extension_key(svc_manufacturer, svc_type)
                self._register_service(extkey, extcls, service_table)
        return
    
    def _scan_for_device_extensions_under_folder(self, directory, device_table):
        """
            Method that scans a code container and its descendants for UpnpRootDevice objects.
        """
        extcoll = collect_extensions_under_folder(directory, UpnpRootDevice)
        for _, extcls in extcoll:
            if hasattr(extcls, "MANUFACTURER") and hasattr(extcls, "MODEL_NUMBER") and hasattr(extcls, "MODEL_DESCRIPTION"):
                extkey = generate_extension_key(getattr(extcls, "MANUFACTURER"),
                    getattr(extcls, "MODEL_NUMBER"), getattr(extcls, "MODEL_DESCRIPTION"))
                self._register_root_device(extkey, extcls, device_table)
        return

    def _scan_for_service_extensions_under_folder(self, directory, service_table):
        """
            Method that scans a code container and its descendants for UpnpServiceProxy objects.
        """
        extcoll = collect_extensions_under_folder(directory, UpnpServiceProxy)
        for _, extcls in extcoll:
            if (hasattr(extcls, "SERVICE_MANUFACTURER") and hasattr(extcls, "SERVICE_TYPE")):
                svc_manufacturer = getattr(extcls, "SERVICE_MANUFACTURER")
                svc_type = getattr(extcls, "SERVICE_TYPE")
                extkey = generate_extension_key(svc_manufacturer, svc_type)
                self._register_service(extkey, extcls, service_table)
        return



"""
.. module:: dnspointer
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Contains the DnsService object which represents a DNS service record.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from typing import Union, TYPE_CHECKING

import socket
import threading

from akit.interop.dns.dnsquestion import DnsQuestion
from akit.interop.dns.dnspacketreader import DnsPacketReader
from akit.networking.constants import MDNS_GROUP_ADDR, MDNS_GROUP_ADDR6, MDNS_PORT

if TYPE_CHECKING:
    from akit.interop.dns.dnspacketwriter import DnsPacketWriter

class DnsServer:
    """
        A DNS server
    """

    def __init__(self, endpoint=(MDNS_GROUP_ADDR, MDNS_PORT) ) -> None:
        self._svc_addr, self._svc_port = endpoint
        
        self._thread_response = None
        self._running = False
        self._shutdown_gate = threading.Event()
        self._shutdown_gate.set()
        return

    @property
    def svc_addr(self):
        return self._svc_addr

    @property
    def svc_port(self):
        return self._svc_port

    def start(self):
        sgate = threading.Event()
        sgate.clear()

        self._thread_response = threading.Thread(name="dns-responder", target=self._entry_thread_response, args=[sgate])
        self._thread_response.setDaemon(True)
        self._thread_response.start()

        sgate.wait()

        return
    
    def query_all_services(self):

        qpacket = DnsQuestion()

        return

    def wait(self, timeout=None):
        
        if self._running:
            self._shutdown_gate.wait(timeout)

    def _entry_thread_response(self, sgate: threading.Event):
        """
            The entry point for monitor thread.  The monitor thread sets up monitoring on all interfaces.

            :param sgate: The startup gate that is used to indicate to the thread spinning up the coordinator
                          that the monitor thread has started.
        """
        multicast_address = MDNS_GROUP_ADDR
        multicast_port = MDNS_PORT

        try:
            self._running = True
            self._shutdown_gate.clear()

            # Set the start gate to allow the thread spinning us up to continue
            sgate.set()

            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)

            try:
                # Make sure other Automation processes can also bind to the UPNP address and port
                # so they can also get responses.
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)

                # Set us up to be a member of the group, this allows us to receive all the packets
                # that are sent to the group
                sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, socket.inet_aton(multicast_address) + socket.inet_aton('0.0.0.0'))

                sock.bind((multicast_address, multicast_port))

                while self._running:
                    msg_data, addr = sock.recvfrom(1024)

                    reader = DnsPacketReader(msg_data)

                    print(reader)

            finally:
                sock.close()

        finally:
            self._shutdown_gate.set()

        return



"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class RadiusClient1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:RadiusClient:1' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'RadiusClient1'

    SERVICE_DEFAULT_VARIABLES = {
        "AuthenticationServerIPAddress": { "data_type": "string", "default": None, "allowed_list": None},
        "AuthenticationServerPortNumber": { "data_type": "ui2", "default": None, "allowed_list": None},
        "AuthenticationServerSharedSecret": { "data_type": "string", "default": None, "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {
        "NumberOfAuthenticationServerEntries": { "data_type": "ui2", "default": None, "allowed_list": None},
    }

    def action_AddAuthenticationServerEntry(self, NewAuthenticationServerIPAddress, NewAuthenticationServerPortNumber, NewAuthenticationServerSharedSecret, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the AddAuthenticationServerEntry action.
        """
        arguments = {
            "NewAuthenticationServerIPAddress": NewAuthenticationServerIPAddress,
            "NewAuthenticationServerPortNumber": NewAuthenticationServerPortNumber,
            "NewAuthenticationServerSharedSecret": NewAuthenticationServerSharedSecret,
        }

        self.call_action("AddAuthenticationServerEntry", arguments=arguments, aspects=aspects)

        return

    def action_DeleteAuthenticationServerEntry(self, NewAuthenticationServerIPAddress, NewAuthenticationServerPortNumber, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the DeleteAuthenticationServerEntry action.
        """
        arguments = {
            "NewAuthenticationServerIPAddress": NewAuthenticationServerIPAddress,
            "NewAuthenticationServerPortNumber": NewAuthenticationServerPortNumber,
        }

        self.call_action("DeleteAuthenticationServerEntry", arguments=arguments, aspects=aspects)

        return

    def action_FactoryDefaultReset(self, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the FactoryDefaultReset action.
        """
        arguments = { }

        self.call_action("FactoryDefaultReset", arguments=arguments, aspects=aspects)

        return

    def action_GetGenericAuthenticationServerEntry(self, NewAuthenticationServerIndex, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetGenericAuthenticationServerEntry action.

            :returns: "NewAuthenticationServerIPAddress", "NewAuthenticationServerPortNumber", "NewAuthenticationServerSharedSecret"
        """
        arguments = {
            "NewAuthenticationServerIndex": NewAuthenticationServerIndex,
        }

        out_params = self.call_action("GetGenericAuthenticationServerEntry", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("NewAuthenticationServerIPAddress", "NewAuthenticationServerPortNumber", "NewAuthenticationServerSharedSecret",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSpecificAuthenticationServerEntry(self, NewAuthenticationServerIPAddress, NewAuthenticationServerPortNumber, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSpecificAuthenticationServerEntry action.

            :returns: "NewAuthenticationServerSharedSecret"
        """
        arguments = {
            "NewAuthenticationServerIPAddress": NewAuthenticationServerIPAddress,
            "NewAuthenticationServerPortNumber": NewAuthenticationServerPortNumber,
        }

        out_params = self.call_action("GetSpecificAuthenticationServerEntry", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("NewAuthenticationServerSharedSecret",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_ResetAuthentication(self, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ResetAuthentication action.
        """
        arguments = { }

        self.call_action("ResetAuthentication", arguments=arguments, aspects=aspects)

        return



"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class DigitalSecurityCameraMotionImage1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:DigitalSecurityCameraMotionImage:1' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'DigitalSecurityCameraMotionImage1'

    SERVICE_DEFAULT_VARIABLES = {
        "AvailableCompressionLevels": { "data_type": "string", "default": None, "allowed_list": None},
        "AvailableEncodings": { "data_type": "string", "default": None, "allowed_list": None},
        "AvailableResolutions": { "data_type": "string", "default": None, "allowed_list": None},
        "VideoPresentationURL": { "data_type": "string", "default": None, "allowed_list": None},
        "VideoURL": { "data_type": "string", "default": None, "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {
        "DefaultCompressionLevel": { "data_type": "string", "default": None, "allowed_list": None},
        "DefaultEncoding": { "data_type": "string", "default": None, "allowed_list": None},
        "DefaultResolution": { "data_type": "string", "default": None, "allowed_list": None},
        "MaxBandwidth": { "data_type": "ui4", "default": "100", "allowed_list": None},
        "TargetFrameRate": { "data_type": "ui4", "default": "1500", "allowed_list": None},
    }

    def action_GetAvailableCompressionLevels(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAvailableCompressionLevels action.

            :returns: "RetAvailableCompressionLevels"
        """
        arguments = { }

        out_params = self.call_action("GetAvailableCompressionLevels", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetAvailableCompressionLevels",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAvailableEncodings(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAvailableEncodings action.

            :returns: "RetAvailableEncodings"
        """
        arguments = { }

        out_params = self.call_action("GetAvailableEncodings", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetAvailableEncodings",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAvailableResolutions(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAvailableResolutions action.

            :returns: "RetAvailableResolutions"
        """
        arguments = { }

        out_params = self.call_action("GetAvailableResolutions", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetAvailableResolutions",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDefaultCompressionLevel(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDefaultCompressionLevel action.

            :returns: "RetCompressionLevel"
        """
        arguments = { }

        out_params = self.call_action("GetDefaultCompressionLevel", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetCompressionLevel",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDefaultEncoding(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDefaultEncoding action.

            :returns: "RetEncoding"
        """
        arguments = { }

        out_params = self.call_action("GetDefaultEncoding", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetEncoding",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDefaultResolution(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDefaultResolution action.

            :returns: "RetResolution"
        """
        arguments = { }

        out_params = self.call_action("GetDefaultResolution", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetResolution",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDefaultVideoPresentationURL(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDefaultVideoPresentationURL action.

            :returns: "RetVideoPresentationURL"
        """
        arguments = { }

        out_params = self.call_action("GetDefaultVideoPresentationURL", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetVideoPresentationURL",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDefaultVideoURL(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDefaultVideoURL action.

            :returns: "RetVideoURL"
        """
        arguments = { }

        out_params = self.call_action("GetDefaultVideoURL", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetVideoURL",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetMaxBandwidth(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetMaxBandwidth action.

            :returns: "RetMaxBandwidth"
        """
        arguments = { }

        out_params = self.call_action("GetMaxBandwidth", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetMaxBandwidth",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTargetFrameRate(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTargetFrameRate action.

            :returns: "RetTargetFrameRate"
        """
        arguments = { }

        out_params = self.call_action("GetTargetFrameRate", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetTargetFrameRate",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetVideoPresentationURL(self, ReqEncoding, ReqCompression, ReqResolution, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetVideoPresentationURL action.

            :returns: "RetVideoPresentationURL"
        """
        arguments = {
            "ReqEncoding": ReqEncoding,
            "ReqCompression": ReqCompression,
            "ReqResolution": ReqResolution,
        }

        out_params = self.call_action("GetVideoPresentationURL", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetVideoPresentationURL",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetVideoURL(self, ReqEncoding, ReqCompression, ReqResolution, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetVideoURL action.

            :returns: "RetVideoURL"
        """
        arguments = {
            "ReqEncoding": ReqEncoding,
            "ReqCompression": ReqCompression,
            "ReqResolution": ReqResolution,
        }

        out_params = self.call_action("GetVideoURL", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RetVideoURL",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SetDefaultCompressionLevel(self, ReqCompressionLevel, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetDefaultCompressionLevel action.
        """
        arguments = {
            "ReqCompressionLevel": ReqCompressionLevel,
        }

        self.call_action("SetDefaultCompressionLevel", arguments=arguments, aspects=aspects)

        return

    def action_SetDefaultEncoding(self, ReqEncoding, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetDefaultEncoding action.
        """
        arguments = {
            "ReqEncoding": ReqEncoding,
        }

        self.call_action("SetDefaultEncoding", arguments=arguments, aspects=aspects)

        return

    def action_SetDefaultResolution(self, ReqResolution, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetDefaultResolution action.
        """
        arguments = {
            "ReqResolution": ReqResolution,
        }

        self.call_action("SetDefaultResolution", arguments=arguments, aspects=aspects)

        return

    def action_SetMaxBandwidth(self, ReqMaxBandwidth, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetMaxBandwidth action.
        """
        arguments = {
            "ReqMaxBandwidth": ReqMaxBandwidth,
        }

        self.call_action("SetMaxBandwidth", arguments=arguments, aspects=aspects)

        return

    def action_SetTargetFrameRate(self, ReqTargetFrameRate, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetTargetFrameRate action.
        """
        arguments = {
            "ReqTargetFrameRate": ReqTargetFrameRate,
        }

        self.call_action("SetTargetFrameRate", arguments=arguments, aspects=aspects)

        return



"""
.. module:: poweragents
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Contains the PowerAgent object instances used to communicate with power controllers.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

import weakref


class DliPowerAgent:

    def __init__(self, interface, attachment):
        self._interface = interface
        self._attachment = attachment
        return

    def on(self):
        success = self._interface.on(outlet=self._attachment)
        return success

    def off(self):
        success = self._interface.off(outlet=self._attachment)
        return success



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

import argparse
import os
import sys

import click

from akit.environment.variables import LOG_LEVEL_NAMES

@click.group("automation")
def group_akit_databases_automation():
    return

HELP_PROFILE = "The name of the configuration profile to use for database connection information."
HELP_SAMPLE_DATA = "Flag indicating if the database should be populated with sample data."

@click.command("create")
@click.option("--profile", show_default="automationdb-admin", default="automationdb-admin", type=str, help=HELP_PROFILE)
@click.option("--sample-data", is_flag=True, show_default=False, default=False, help=HELP_SAMPLE_DATA)
def command_akit_databases_automation_create(profile, sample_data):

    import akit.activation.console

    from akit.datum.dbio import create_automation_database

    create_automation_database(profile)

    return

@click.command("reset")
@click.option("--profile", show_default="automationdb-admin", default="automationdb-admin", type=str, help=HELP_PROFILE)
def command_akit_databases_automation_reset(profile):

    import akit.activation.console

    from akit.datum.dbio import reset_automation_database

    reset_automation_database(profile)

    return

group_akit_databases_automation.add_command(command_akit_databases_automation_create)
group_akit_databases_automation.add_command(command_akit_databases_automation_reset)



"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class AudioIn1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'AudioIn1' service.
    """

    SERVICE_MANUFACTURER = 'SonosInc'
    SERVICE_TYPE = 'urn:schemas-upnp-org:service:AudioIn:1'

    SERVICE_DEFAULT_VARIABLES = {}

    SERVICE_EVENT_VARIABLES = {
        "AudioInputName": { "data_type": "string", "default": None, "allowed_list": None},
        "Icon": { "data_type": "string", "default": None, "allowed_list": None},
        "LeftLineInLevel": { "data_type": "i4", "default": None, "allowed_list": None},
        "LineInConnected": { "data_type": "boolean", "default": None, "allowed_list": None},
        "Playing": { "data_type": "boolean", "default": None, "allowed_list": None},
        "RightLineInLevel": { "data_type": "i4", "default": None, "allowed_list": None},
    }

    def action_GetAudioInputAttributes(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAudioInputAttributes action.

            :returns: "CurrentName", "CurrentIcon"
        """
        arguments = { }

        out_params = self.call_action("GetAudioInputAttributes", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentName", "CurrentIcon",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetLineInLevel(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetLineInLevel action.

            :returns: "CurrentLeftLineInLevel", "CurrentRightLineInLevel"
        """
        arguments = { }

        out_params = self.call_action("GetLineInLevel", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentLeftLineInLevel", "CurrentRightLineInLevel",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SelectAudio(self, ObjectID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SelectAudio action.
        """
        arguments = {
            "ObjectID": ObjectID,
        }

        self.call_action("SelectAudio", arguments=arguments, aspects=aspects)

        return

    def action_SetAudioInputAttributes(self, DesiredName, DesiredIcon, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetAudioInputAttributes action.
        """
        arguments = {
            "DesiredName": DesiredName,
            "DesiredIcon": DesiredIcon,
        }

        self.call_action("SetAudioInputAttributes", arguments=arguments, aspects=aspects)

        return

    def action_SetLineInLevel(self, DesiredLeftLineInLevel, DesiredRightLineInLevel, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetLineInLevel action.
        """
        arguments = {
            "DesiredLeftLineInLevel": DesiredLeftLineInLevel,
            "DesiredRightLineInLevel": DesiredRightLineInLevel,
        }

        self.call_action("SetLineInLevel", arguments=arguments, aspects=aspects)

        return

    def action_StartTransmissionToGroup(self, CoordinatorID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StartTransmissionToGroup action.

            :returns: "CurrentTransportSettings"
        """
        arguments = {
            "CoordinatorID": CoordinatorID,
        }

        out_params = self.call_action("StartTransmissionToGroup", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentTransportSettings",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_StopTransmissionToGroup(self, CoordinatorID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StopTransmissionToGroup action.
        """
        arguments = {
            "CoordinatorID": CoordinatorID,
        }

        self.call_action("StopTransmissionToGroup", arguments=arguments, aspects=aspects)

        return




from typing import Callable, Union

from akit.testing.testplus.resourcelifespan import ResourceLifespan

from akit.testing.testplus.scopecoupling import ScopeCoupling
from akit.testing.testplus.registration.sourcebase import SourceBase

class ScopeSource(SourceBase):

    def __init__(self, source_func: Callable, query_func: Union[Callable, None], scope_type: ScopeCoupling, constraints: dict):
        SourceBase.__init__(self, source_func, query_func, scope_type, constraints)
        self._source_func = source_func
        return



"""
.. module:: coordinatorcoupling
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Module containing the :class:`CoordinatorCoupling` class and associated reflection methods.
        The :class:`CoordinatorCoupling` derived classes can be used to integraton automation resources and roles
        into the test environment.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>
"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from typing import TYPE_CHECKING

import inspect

from akit.exceptions import AKitNotOverloadedError
from akit.coupling.integrationcoupling import IntegrationCoupling

# Types imported only for type checking purposes
if TYPE_CHECKING:
    from akit.interop.landscaping.landscape import Landscape

class CoordinatorCoupling(IntegrationCoupling):
    """
        The :class:`CoordinatorCoupling` object serves as the base object for the declaration of an
        automation integration coordinator coupling.
    """

    coordinator = None

    @classmethod
    def create_coordinator(cls, landscape: "Landscape") -> object:
        """
            This API is called so that the landscape can create a coordinator for a given coordinator
            integration role.
        """
        raise AKitNotOverloadedError("This method must be overridden by derived coordinator classes.") from None



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

class DiagnosticLabel:
    PRERUN_DIAGNOSTIC = "prerun-diagnostic"
    POSTRUN_DIAGNOSTIC = "postrun-diagnostic"

class RuntimeConfigPaths:
    DIAGNOSTIC_PRERUN = "/configuration/diagnostics/prerun-diagnostic"
    DIAGNOSTIC_POSTRUN = "/configuration/diagnostics/postrun-diagnostic"



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from akit.exceptions import AKitValueError

# NOTE: This file needs to be importable very early in the startup process
# so don't do any logging, use any external resources or dependencies
# in here.  The state of all functions should be threadsafe and self contained.
# The functions should only use what is passed in, process it, and provide an
# output.

STRINGS_FOR_FALSE = [
    "0",
    "FALSE",
    "NO",
    "OFF"
]

STRINGS_FOR_TRUE = [
    "1",
    "TRUE",
    "YES",
    "ON"
]

def string_to_bool(sval: str) -> bool:
    """
        Converts a string value to a boolean value.

        :returns: Coverted boolean result.
    """
    bval = None

    sval = sval.upper()
    if sval in STRINGS_FOR_FALSE:
        bval = False
    elif sval in STRINGS_FOR_TRUE:
        bval = True
    else:
        raise AKitValueError("Invalid parameter, unable to convert '{}' to bool.".format(sval))

    return bval



__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"

from distutils.sysconfig import PREFIX
from typing import Any, List, Optional, Union, Type

import os
import re

from pprint import pformat

from akit.xformatting import indent_lines

from akit.exceptions import AKitAssertionError, TracebackFormatPolicy

__traceback_format_policy__ = TracebackFormatPolicy.Hide

PREFIX_STD = "Result"
PREFIX_API = "'{}' API result"

def assert_dict_response_has_keys(to_inspect: dict, expected_keys: List[str], api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected keys.  If the
        verification fails then an :class:`AKitAssertionError` is created and return, otherwise None is returned. It
        is the resposibility of the calling test to raise the returned error.

        :param to_inspect: The dictionary being inspected
        :param expected_keys: The list of expected keys
        :param api: The optional name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    template = "    '{}' key not found."

    content_errors = verify_dict_has_keys(to_inspect, expected_keys, template)
    if len(content_errors) > 0:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed with the following errors:".format(msg_prefix)
        ]

        for nxtce in content_errors:
            err_msg_lines.append(nxtce)

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_dict_response_has_paths(to_inspect: dict, expected_paths: List[str], api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected paths.  If the
        verification fails then an :class:`AKitAssertionError` is created and return, otherwise None is returned. It
        is the resposibility of the calling test to raise the returned error.

        :param to_inspect: The dictionary being inspected
        :param expected_keys: The list of expected keys
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    template = "    '{}' path not found."

    content_errors = verify_dict_has_paths(to_inspect, expected_paths, template)
    if len(content_errors) > 0:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed with the following errors:".format(msg_prefix)
        ]

        for nxtce in content_errors:
            err_msg_lines.append(nxtce)

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return


def assert_equal(found: Any, expected: Any, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected value.
        If the verification fails then an :class:`AKitAssertionError` is created and returned, otherwise None
        is returned. It is the resposibility of the calling test to raise the returned error.

        :param found: The value to be compared
        :param expected: The expected value
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    if found != expected:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the found value did not match expected value:".format(msg_prefix),
            "EXPECTED:"
        ]

        exp_format_lines = pformat(expected, indent=4).strip().splitlines()
        if len(exp_format_lines) == 1:
            err_msg_lines.append("EXPECTED: {}".format(exp_format_lines[0]))
        elif len(exp_format_lines) > 1:
            err_msg_lines.append("EXPECTED:")
            exp_format_lines = indent_lines(exp_format_lines, 1)
            err_msg_lines.extend(exp_format_lines)

        found_format_lines = pformat(found, indent=4).splitlines()
        if len(found_format_lines) == 1:
            err_msg_lines.append("FOUND: {}".format(found_format_lines[0]))
        elif len(found_format_lines) > 1:
            err_msg_lines.append("FOUND:")
            found_format_lines = indent_lines(found_format_lines, 1)
            err_msg_lines.extend(found_format_lines)

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_expression(found: str, expr: Union[str, re.Pattern], api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected value.
        If the verification fails then an :class:`AKitAssertionError` is created and returned, otherwise None
        is returned. It is the resposibility of the calling test to raise the returned error.

        :param found: The value to be compared
        :param expr: An expression to match with the found value.
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    if not isinstance(expr, re.Pattern):
        expr = re.compile(expr)

    mobj = expr.match(found)
    if mobj is None:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the found value did not match expression provided.".format(msg_prefix),
            "FOUND: {}".format(found),
            "EXPR: {}".format(expr)
        ]

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_greater(found: Any, boundary: Any, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has a value greater than the
        boundary specified. If the verification fails then an :class:`AKitAssertionError` is created
        and returned, otherwise None is returned. It is the resposibility of the calling test to raise
        the returned error.

        :param found: The value being compared
        :param boundry: The boundary value being compared against
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    if not found > boundary:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the found value was not greater than the boundary value:".format(msg_prefix)
        ]

        bndry_format_lines = pformat(boundary, indent=4).strip().splitlines()
        if len(bndry_format_lines) == 1:
            err_msg_lines.append("BOUNDARY: {}".format(bndry_format_lines[0]))
        elif len(bndry_format_lines) > 1:
            err_msg_lines.append("BOUNDARY:")
            bndry_format_lines = indent_lines(bndry_format_lines, 1)
            err_msg_lines.extend(bndry_format_lines)

        found_format_lines = pformat(found, indent=4).splitlines()
        if len(found_format_lines) == 1:
            err_msg_lines.append("FOUND: {}".format(found_format_lines[0]))
        elif len(found_format_lines) > 1:
            err_msg_lines.append("FOUND:")
            found_format_lines = indent_lines(found_format_lines, 1)
            err_msg_lines.extend(found_format_lines)

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_lessthan(found: Any, boundary: Any, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has a value less than the
        boundary specified. If the verification fails then an :class:`AKitAssertionError` is created
        and returned, otherwise None is returned. It is the resposibility of the calling test to raise
        the returned error.

        :param found: The value being compared
        :param boundry: The boundary value being compared against
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    if not found < boundary:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the found value was not less than the boundary value:".format(msg_prefix)
        ]

        bndry_format_lines = pformat(boundary, indent=4).strip().splitlines()
        if len(bndry_format_lines) == 1:
            err_msg_lines.append("BOUNDARY: {}".format(bndry_format_lines[0]))
        elif len(bndry_format_lines) > 1:
            err_msg_lines.append("BOUNDARY:")
            bndry_format_lines = indent_lines(bndry_format_lines, 1)
            err_msg_lines.extend(bndry_format_lines)

        found_format_lines = pformat(found, indent=4).splitlines()
        if len(found_format_lines) == 1:
            err_msg_lines.append("FOUND: {}".format(found_format_lines[0]))
        elif len(found_format_lines) > 1:
            err_msg_lines.append("FOUND:")
            found_format_lines = indent_lines(found_format_lines, 1)
            err_msg_lines.extend(found_format_lines)

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return


def assert_list_length(to_inspect: List, expected_len: int, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected number of items.
        If the verification fails then an :class:`AKitAssertionError` is created and returned, otherwise None
        is returned. It is the resposibility of the calling test to raise the returned error.

        :param to_inspect: The list being inspected
        :param expected_len: The expected length of the list
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    found_len = len(to_inspect)
    if found_len != expected_len:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the actual length ({}) did not match expected length ({}):".format(
                msg_prefix, found_len, expected_len),
            "ITEMS:"
        ]

        for fitem in to_inspect:
            err_msg_lines.append("    {}".format(fitem))
        err_msg_lines.append("")

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_list_length_greater(to_inspect: List, boundary_len: int, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has more items than the specified expected
        number of items.  If the verification fails then an :class:`AKitAssertionError` is created and returned,
        otherwise None is returned. It is the resposibility of the calling test to raise the returned error.

        :param to_inspect: The list being inspected
        :param boundary_len: The boundary that the list length should exceed.
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    found_len = len(to_inspect)
    if not found_len > boundary_len:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the actual length ({}) was not greater than the boundary length ({}):".format(
                msg_prefix, found_len, boundary_len),
            "ITEMS:"
        ]

        for fitem in to_inspect:
            err_msg_lines.append("    {}".format(fitem))
        err_msg_lines.append("")

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return

def assert_list_length_less(to_inspect: List, boundary_len: int, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has less items than the specified expected
        number of items.  If the verification fails then an :class:`AKitAssertionError` is created and returned,
        otherwise None is returned. It is the resposibility of the calling test to raise the returned error.

        :param to_inspect: The list being inspected
        :param boundary_len: The boundary the list length should not exceed
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """
    
    found_len = len(to_inspect)
    if not found_len < boundary_len:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)
        
        err_msg_lines = [
            "{} result verification failed because the actual length ({}) was not less than the boundary length ({}):".format(
                msg_prefix, found_len, boundary_len),
            "ITEMS:"
        ]

        for fitem in to_inspect:
            err_msg_lines.append("    {}".format(fitem))
        err_msg_lines.append("")

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return


def assert_type(found: Any, exp_type: Type, api: Optional[str] = None):
    """
        Verifies that the specified return result from the specified API has the specified expected value type.
        If the verification fails then an :class:`AKitAssertionError` is created and returned, otherwise None
        is returned. It is the resposibility of the calling test to raise the returned error.

        :param found: The value to check the type of
        :param exp_type: The expected value type
        :param api: The name of the API that returned the result being inspected.
        
        :returns: None or an :class:`AKitAssertionError` for the caller to raise.
    """

    found_type = type(found)
    if found_type != exp_type:

        msg_prefix = PREFIX_STD if api is None else PREFIX_API.format(api)

        err_msg_lines = [
            "{} verification failed because the found value did not the expected type={}.".format(msg_prefix, exp_type),
            "FOUND_TYPE: {}".format(found_type)
        ]

        errmsg = os.linesep.join(err_msg_lines)
        raise AKitAssertionError(errmsg)

    return


def verify_dict_has_keys(to_inspect: dict, expected_keys: List[str], template: Optional[str]=None) -> List[str]:
    """
        Verifies that the expected keys are found in the dictionary provided.  Returns a list of
        any keys that are missing.

        :param to_inspect: The dictionary being inspected
        :param expected_keys: The list of expected keys
        :param template: Optional template to combine with the missing key names to
                         product error messages for missing keys.

        :returns: List of missing keys or error messages.
    """

    missing_list = []

    if template is not None:
        for exkey in expected_keys:
            if exkey not in to_inspect:
                missing_list.append(template.format(exkey))
    else:
        for exkey in expected_keys:
            if exkey not in to_inspect:
                missing_list.append(exkey)

    return missing_list

def verify_dict_has_paths(to_inspect: dict, expected_paths: List[str], template: Optional[str]=None) -> List[str]:
    """
        Verifies that the expected keys are found in the dictionary provided.  Returns a list of
        any keys that are missing.

        :param to_inspect: The dictionary being inspected
        :param expected_paths: The list of expected paths to leafs in a dictionary
        :param template: Optional template to combine with the missing key names to
                         product error messages for missing keys.

        :returns: List of missing keys or error messages.
    """

    missing_list = []

    for next_exp_path in expected_paths:
        next_exp_path = next_exp_path.strip("/")

        leaf_parts = []

    return missing_list

def _verify_dict_has_paths_descend(search_path: str, cursor: dict, leaf_parts: list):

    found = False

    cursor_path = "/".join(leaf_parts)

    for nxt_key in cursor.keys():
        nxt_path = cursor_path + "/" + nxt_key
        if nxt_path == search_path:
            found = True
        elif search_path.startswith(nxt_path):
            nxt_val = cursor[nxt_key]
            if isinstance(nxt_val, dict):
                desc_leaf_parts = [lp for lp in leaf_parts]
                desc_leaf_parts.extend(nxt_key)
                _verify_dict_has_paths_descend(search_path, nxt_val, desc_leaf_parts)

    return found




"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class ApplicationManagement2ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:ApplicationManagement:2' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'ApplicationManagement2'

    SERVICE_DEFAULT_VARIABLES = {
        "AppInfoList": { "data_type": "string", "default": None, "allowed_list": None},
        "FeatureList": { "data_type": "string", "default": None, "allowed_list": None},
        "SupportedTargetFields": { "data_type": "string", "default": None, "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {
        "RunningAppList": { "data_type": "string", "default": None, "allowed_list": None},
        "TransitioningApps": { "data_type": "string", "default": None, "allowed_list": None},
    }

    def action_ConnectApptoApp(self, AppID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the ConnectApptoApp action.

            :returns: "ConnectionID"
        """
        arguments = {
            "AppID": AppID,
        }

        out_params = self.call_action("ConnectApptoApp", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("ConnectionID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_DisconnectApptoApp(self, ConnectionIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the DisconnectApptoApp action.

            :returns: "DisconnectedConnectionIDs"
        """
        arguments = {
            "ConnectionIDs": ConnectionIDs,
        }

        out_params = self.call_action("DisconnectApptoApp", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DisconnectedConnectionIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAppConnectionInfo(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAppConnectionInfo action.

            :returns: "ConnectionInfo"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("GetAppConnectionInfo", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("ConnectionInfo",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAppIDList(self, Target, TargetFields, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAppIDList action.

            :returns: "AppIDs"
        """
        arguments = {
            "Target": Target,
            "TargetFields": TargetFields,
        }

        out_params = self.call_action("GetAppIDList", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AppIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetAppInfoByIDs(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetAppInfoByIDs action.

            :returns: "AppInfo"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("GetAppInfoByIDs", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AppInfo",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetCurrentConnectionInfo(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetCurrentConnectionInfo action.

            :returns: "ConnectionIDs", "ConnectionAppIDs"
        """
        arguments = { }

        out_params = self.call_action("GetCurrentConnectionInfo", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("ConnectionIDs", "ConnectionAppIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFeatureList(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFeatureList action.

            :returns: "FeatureList"
        """
        arguments = { }

        out_params = self.call_action("GetFeatureList", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("FeatureList",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetInstallationStatus(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetInstallationStatus action.

            :returns: "InstallationStatus"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("GetInstallationStatus", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("InstallationStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetRunningAppList(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetRunningAppList action.

            :returns: "RunningAppList"
        """
        arguments = { }

        out_params = self.call_action("GetRunningAppList", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RunningAppList",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetRunningStatus(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetRunningStatus action.

            :returns: "RunningStatus"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("GetRunningStatus", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RunningStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSupportedTargetFields(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSupportedTargetFields action.

            :returns: "SupportedTargetFields"
        """
        arguments = { }

        out_params = self.call_action("GetSupportedTargetFields", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("SupportedTargetFields",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_InstallAppByID(self, AppID, InstallParameters, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the InstallAppByID action.
        """
        arguments = {
            "AppID": AppID,
            "InstallParameters": InstallParameters,
        }

        self.call_action("InstallAppByID", arguments=arguments, aspects=aspects)

        return

    def action_InstallAppByURI(self, InstallationURI, AppInfo, InstallParameters, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the InstallAppByURI action.

            :returns: "AppID"
        """
        arguments = {
            "InstallationURI": InstallationURI,
            "AppInfo": AppInfo,
            "InstallParameters": InstallParameters,
        }

        out_params = self.call_action("InstallAppByURI", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AppID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_StartAppByID(self, AppID, StartParameters, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StartAppByID action.
        """
        arguments = {
            "AppID": AppID,
            "StartParameters": StartParameters,
        }

        self.call_action("StartAppByID", arguments=arguments, aspects=aspects)

        return

    def action_StartAppByURI(self, StartURI, AppInfo, StartParameters, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StartAppByURI action.

            :returns: "AppID"
        """
        arguments = {
            "StartURI": StartURI,
            "AppInfo": AppInfo,
            "StartParameters": StartParameters,
        }

        out_params = self.call_action("StartAppByURI", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("AppID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_StopApp(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the StopApp action.

            :returns: "StoppedAppIDs"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("StopApp", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("StoppedAppIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_UninstallApp(self, AppIDs, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the UninstallApp action.

            :returns: "UninstalledAppIDs"
        """
        arguments = {
            "AppIDs": AppIDs,
        }

        out_params = self.call_action("UninstallApp", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("UninstalledAppIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args



"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class FanSpeed1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:FanSpeed:1' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'FanSpeed1'

    SERVICE_DEFAULT_VARIABLES = {
        "DirectionStatus": { "data_type": "boolean", "default": "0", "allowed_list": None},
        "DirectionTarget": { "data_type": "boolean", "default": "0", "allowed_list": None},
        "FanSpeedStatus": { "data_type": "ui1", "default": "0", "allowed_list": None},
        "FanSpeedTarget": { "data_type": "ui1", "default": "0", "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {}

    def action_GetFanDirection(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFanDirection action.

            :returns: "CurrentDirectionStatus"
        """
        arguments = { }

        out_params = self.call_action("GetFanDirection", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentDirectionStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFanDirectionTarget(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFanDirectionTarget action.

            :returns: "CurrentDirectionTarget"
        """
        arguments = { }

        out_params = self.call_action("GetFanDirectionTarget", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentDirectionTarget",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFanSpeed(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFanSpeed action.

            :returns: "CurrentFanSpeedStatus"
        """
        arguments = { }

        out_params = self.call_action("GetFanSpeed", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentFanSpeedStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetFanSpeedTarget(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetFanSpeedTarget action.

            :returns: "CurrentFanSpeedTarget"
        """
        arguments = { }

        out_params = self.call_action("GetFanSpeedTarget", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("CurrentFanSpeedTarget",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SetFanDirection(self, NewDirectionTarget, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetFanDirection action.
        """
        arguments = {
            "NewDirectionTarget": NewDirectionTarget,
        }

        self.call_action("SetFanDirection", arguments=arguments, aspects=aspects)

        return

    def action_SetFanSpeed(self, NewFanSpeedTarget, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetFanSpeed action.
        """
        arguments = {
            "NewFanSpeedTarget": NewFanSpeedTarget,
        }

        self.call_action("SetFanSpeed", arguments=arguments, aspects=aspects)

        return



"""
.. module:: paths
    :platform: Darwin, Linux, Unix, Windows
    :synopsis: Module containing constants for different paths utilize by the code generator that are relative to the test framework.

.. moduleauthor:: Myron Walker <myron.walker@gmail.com>

"""

__author__ = "Myron Walker"
__copyright__ = "Copyright 2020, Myron W Walker"
__credits__ = []
__version__ = "1.0.0"
__maintainer__ = "Myron Walker"
__email__ = "myron.walker@gmail.com"
__status__ = "Development" # Prototype, Development or Production
__license__ = "MIT"


import os

from akit.environment.variables import AKIT_VARIABLES

DIR_UPNP = os.path.dirname(__file__)


DIR_UPNP_EXTENSIONS = os.path.join(DIR_UPNP, "extensions")

DIR_UPNP_EXTENSIONS_STANDARD = os.path.join(DIR_UPNP_EXTENSIONS, "standard")
DIR_UPNP_EXTENSIONS_STANDARD_EMBEDDEDDEVICES = os.path.join(DIR_UPNP_EXTENSIONS_STANDARD, "embeddeddevices")
DIR_UPNP_EXTENSIONS_STANDARD_ROOTDEVICES = os.path.join(DIR_UPNP_EXTENSIONS_STANDARD, "rootdevices")
DIR_UPNP_EXTENSIONS_STANDARD_SERVICES = os.path.join(DIR_UPNP_EXTENSIONS_STANDARD, "services")

DIR_UPNP_GENERATOR = os.path.join(DIR_UPNP, "generator")

DIR_UPNP_GENERATOR_STANDARD = os.path.join(DIR_UPNP_GENERATOR, "standard")
DIR_UPNP_GENERATOR_STANDARD_EMBEDDEDDEVICES = os.path.join(DIR_UPNP_GENERATOR_STANDARD, "embeddeddevices")
DIR_UPNP_GENERATOR_STANDARD_ROOTDEVICES = os.path.join(DIR_UPNP_GENERATOR_STANDARD, "rootdevices")
DIR_UPNP_GENERATOR_STANDARD_SERVICES = os.path.join(DIR_UPNP_GENERATOR_STANDARD, "services")


DIR_UPNP_EXTENSIONS_INTEGRATION = os.path.join(AKIT_VARIABLES.AKIT_HOME_DIRECTORY, "upnp", "extensions")
if AKIT_VARIABLES.AKIT_UPNP_EXTENSIONS_INTEGRATION_BASE is not None:
    DIR_UPNP_EXTENSIONS_INTEGRATION = AKIT_VARIABLES.AKIT_UPNP_EXTENSIONS_INTEGRATION_BASE

DIR_UPNP_EXTENSIONS_INTEGRATION_EMBEDDEDDEVICES = os.path.join(DIR_UPNP_EXTENSIONS_INTEGRATION, "embeddeddevices")
DIR_UPNP_EXTENSIONS_INTEGRATION_ROOTDEVICES = os.path.join(DIR_UPNP_EXTENSIONS_INTEGRATION, "rootdevices")
DIR_UPNP_EXTENSIONS_INTEGRATION_SERVICES = os.path.join(DIR_UPNP_EXTENSIONS_INTEGRATION, "services")

DIR_UPNP_SCAN_INTEGRATION = os.path.join(AKIT_VARIABLES.AKIT_HOME_DIRECTORY, "upnp", "scan")
if AKIT_VARIABLES.AKIT_UPNP_SCAN_INTEGRATION_BASE is not None:
    DIR_UPNP_SCAN_INTEGRATION = AKIT_VARIABLES.AKIT_UPNP_SCAN_INTEGRATION_BASE

DIR_UPNP_SCAN_INTEGRATION_EMBEDDEDDEVICES = os.path.join(DIR_UPNP_SCAN_INTEGRATION, "embeddeddevices")
DIR_UPNP_SCAN_INTEGRATION_ROOTDEVICES = os.path.join(DIR_UPNP_SCAN_INTEGRATION, "rootdevices")
DIR_UPNP_SCAN_INTEGRATION_SERVICES = os.path.join(DIR_UPNP_SCAN_INTEGRATION, "services")



"""

    NOTE: This is a code generated file.  This file should not be edited directly.
"""



from akit.aspects import AspectsUPnP, DEFAULT_UPNP_ASPECTS

from akit.extensible import LoadableExtension
from akit.interop.upnp.services.upnpserviceproxy import UpnpServiceProxy

class BasicManagement1ServiceProxy(UpnpServiceProxy, LoadableExtension):
    """
        This is a code generated proxy class to the 'urn:schemas-upnp-org:service:BasicManagement:1' service.
    """

    SERVICE_MANUFACTURER = 'UPnP'
    SERVICE_TYPE = 'BasicManagement1'

    SERVICE_DEFAULT_VARIABLES = {
        "ActiveTestIDs": { "data_type": "string", "default": None, "allowed_list": None},
        "DeviceStatus": { "data_type": "string", "default": None, "allowed_list": None},
        "LogURIs": { "data_type": "string", "default": None, "allowed_list": None},
        "SequenceMode": { "data_type": "boolean", "default": "0", "allowed_list": None},
    }

    SERVICE_EVENT_VARIABLES = {}

    def action_BaselineReset(self, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the BaselineReset action.
        """
        arguments = { }

        self.call_action("BaselineReset", arguments=arguments, aspects=aspects)

        return

    def action_CancelTest(self, TestID, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the CancelTest action.
        """
        arguments = {
            "TestID": TestID,
        }

        self.call_action("CancelTest", arguments=arguments, aspects=aspects)

        return

    def action_GetActiveTestIDs(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetActiveTestIDs action.

            :returns: "TestIDs"
        """
        arguments = { }

        out_params = self.call_action("GetActiveTestIDs", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestIDs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetDeviceStatus(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetDeviceStatus action.

            :returns: "DeviceStatus"
        """
        arguments = { }

        out_params = self.call_action("GetDeviceStatus", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("DeviceStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetInterfaceResetResult(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetInterfaceResetResult action.

            :returns: "Status", "AdditionalInfo", "NumberOfSuccesses", "NumberOfFailures"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetInterfaceResetResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Status", "AdditionalInfo", "NumberOfSuccesses", "NumberOfFailures",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetLogInfo(self, LogURI, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetLogInfo action.

            :returns: "Configurable", "Enabled", "LogLevel", "LogURL", "MaxSize", "LastChange"
        """
        arguments = {
            "LogURI": LogURI,
        }

        out_params = self.call_action("GetLogInfo", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Configurable", "Enabled", "LogLevel", "LogURL", "MaxSize", "LastChange",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetLogURIs(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetLogURIs action.

            :returns: "LogURIs"
        """
        arguments = { }

        out_params = self.call_action("GetLogURIs", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("LogURIs",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetNSLookupResult(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetNSLookupResult action.

            :returns: "Status", "AdditionalInfo", "SuccessCount", "Result"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetNSLookupResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Status", "AdditionalInfo", "SuccessCount", "Result",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetPingResult(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetPingResult action.

            :returns: "Status", "AdditionalInfo", "SuccessCount", "FailureCount", "AverageResponseTime", "MinimumResponseTime", "MaximumResponseTime"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetPingResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Status", "AdditionalInfo", "SuccessCount", "FailureCount", "AverageResponseTime", "MinimumResponseTime", "MaximumResponseTime",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSelfTestResult(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSelfTestResult action.

            :returns: "Status", "AdditionalInfo"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetSelfTestResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Status", "AdditionalInfo",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetSequenceMode(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetSequenceMode action.

            :returns: "SequenceMode"
        """
        arguments = { }

        out_params = self.call_action("GetSequenceMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("SequenceMode",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTestInfo(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTestInfo action.

            :returns: "Type", "State"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetTestInfo", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Type", "State",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_GetTracerouteResult(self, TestID, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the GetTracerouteResult action.

            :returns: "Status", "AdditionalInfo", "ResponseTime", "HopHosts"
        """
        arguments = {
            "TestID": TestID,
        }

        out_params = self.call_action("GetTracerouteResult", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("Status", "AdditionalInfo", "ResponseTime", "HopHosts",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_InterfaceReset(self, Interfaces, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the InterfaceReset action.

            :returns: "TestID"
        """
        arguments = {
            "Interfaces": Interfaces,
        }

        out_params = self.call_action("InterfaceReset", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_NSLookup(self, HostName, DNSServer, NumberOfRepetitions, Timeout, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the NSLookup action.

            :returns: "TestID"
        """
        arguments = {
            "HostName": HostName,
            "DNSServer": DNSServer,
            "NumberOfRepetitions": NumberOfRepetitions,
            "Timeout": Timeout,
        }

        out_params = self.call_action("NSLookup", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_Ping(self, Host, NumberOfRepetitions, Timeout, DataBlockSize, DSCP, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the Ping action.

            :returns: "TestID"
        """
        arguments = {
            "Host": Host,
            "NumberOfRepetitions": NumberOfRepetitions,
            "Timeout": Timeout,
            "DataBlockSize": DataBlockSize,
            "DSCP": DSCP,
        }

        out_params = self.call_action("Ping", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_Reboot(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the Reboot action.

            :returns: "RebootStatus"
        """
        arguments = { }

        out_params = self.call_action("Reboot", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("RebootStatus",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SelfTest(self, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SelfTest action.

            :returns: "TestID"
        """
        arguments = { }

        out_params = self.call_action("SelfTest", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_SetLogInfo(self, LogURI, Enabled, LogLevel, *, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetLogInfo action.
        """
        arguments = {
            "LogURI": LogURI,
            "Enabled": Enabled,
            "LogLevel": LogLevel,
        }

        self.call_action("SetLogInfo", arguments=arguments, aspects=aspects)

        return

    def action_SetSequenceMode(self, NewSequenceMode, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the SetSequenceMode action.

            :returns: "OldSequenceMode"
        """
        arguments = {
            "NewSequenceMode": NewSequenceMode,
        }

        out_params = self.call_action("SetSequenceMode", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("OldSequenceMode",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args

    def action_Traceroute(self, Host, Timeout, DataBlockSize, MaxHopCount, DSCP, *, extract_returns=True, aspects:AspectsUPnP=DEFAULT_UPNP_ASPECTS):
        """
            Calls the Traceroute action.

            :returns: "TestID"
        """
        arguments = {
            "Host": Host,
            "Timeout": Timeout,
            "DataBlockSize": DataBlockSize,
            "MaxHopCount": MaxHopCount,
            "DSCP": DSCP,
        }

        out_params = self.call_action("Traceroute", arguments=arguments, aspects=aspects)

        rtn_args = out_params
        if extract_returns:
            rtn_args = [out_params[k] for k in ("TestID",)]
            if len(rtn_args) == 1:
                rtn_args = rtn_args[0]

        return rtn_args
